@@ -16,6 +16,8 @@
  */
 extern int hard_math;
 extern int x86;
+extern char x86_vendor_id[13];
+extern fdiv_bug;		
 extern int ignore_irq13;
 extern int wp_works_ok;		/* doesn't work on a 386 */
 extern int hlt_works_ok;	/* problems on some 486Dx4's and old 386's */
@@ -288,8 +290,7 @@ struct task_struct {
 /* file system info */
 	int link_count;
 	struct tty_struct *tty; /* NULL if no tty */
-/* shm stuff */
-	struct shm_desc *shm;
+/* ipc stuff */
 	struct sem_undo *semundo;
 /* ldt for this task - used by Wine.  If NULL, default_ldt is used */
 	struct desc_struct *ldt;
@@ -343,7 +344,7 @@ struct task_struct {
 /* comm */	"swapper", \
 /* vm86_info */	NULL, 0, 0, 0, 0, \
 /* fs info */	0,NULL, \
-/* ipc */	NULL, NULL, \
+/* ipc */	NULL, \
 /* ldt */	NULL, \
 /* tss */	INIT_TSS, \
 /* fs */	{ INIT_FS }, \
